@@ -31,6 +31,7 @@
 #include "ephy-search-engine-manager.h"
 #include "ephy-settings.h"
 #include "ephy-string.h"
+#include "ephy-uri-helpers.h"
 #include "ephy-view-source-handler.h"
 
 #include <glib/gi18n.h>
@@ -329,7 +330,11 @@ ephy_embed_utils_autosearch_address (const char *search_key)
 
   shell = ephy_embed_shell_get_default ();
   manager = ephy_embed_shell_get_search_engine_manager (shell);
-  engine = ephy_search_engine_manager_get_default_engine (manager);
+
+  if (ephy_embed_shell_get_mode (ephy_embed_shell_get_default ()) == EPHY_EMBED_SHELL_MODE_INCOGNITO)
+    engine = ephy_search_engine_manager_get_incognito_engine (manager);
+  else
+    engine = ephy_search_engine_manager_get_default_engine (manager);
   g_assert (engine != NULL);
 
   return ephy_search_engine_build_search_address (engine, search_key);
@@ -388,6 +393,8 @@ ephy_embed_utils_is_no_show_address (const char *address)
 char *
 ephy_embed_utils_get_title_from_address (const char *address)
 {
+  g_autofree char *decoded_url = NULL;
+
   if (g_str_has_prefix (address, "file://"))
     return g_strdup (address + 7);
 
@@ -397,7 +404,11 @@ ephy_embed_utils_get_title_from_address (const char *address)
       !strcmp (address, "about:newtab"))
     return g_strdup (_(NEW_TAB_PAGE_TITLE));
 
-  return ephy_string_get_host_name (address);
+  decoded_url = ephy_uri_decode (address);
+  if (!decoded_url)
+    return g_strdup (address);
+
+  return ephy_uri_get_decoded_host (decoded_url);
 }
 
 void
