@@ -25,10 +25,12 @@
 
 GdkTexture *ephy_texture_new_for_pixbuf (GdkPixbuf *pixbuf);
 
-GdkPixbuf *ephy_texture_to_pixbuf (GdkTexture *texture);
+GdkPixbuf *ephy_texture_to_pixbuf (GdkTexture *texture,
+                                   gboolean has_alpha);
 
 GdkPixbuf *ephy_get_pixbuf_from_surface (cairo_surface_t *surface,
                                          int              src_x,
                                          int              src_y,
                                          int              width,
-                                         int              height);
\ No newline at end of file
+                                         int              height);
+
