@@ -35,6 +35,7 @@ struct _EphySearchEngineManager {
   GPtrArray *engines;
 
   EphySearchEngine *default_engine; /* unowned */
+  EphySearchEngine *incognito_engine; /* unowned */
 
   /* This is just to speed things up. It updates based on each search engine's
    * notify::bang signal, so it is never out of sync because signal callbacks
@@ -54,6 +55,7 @@ G_DEFINE_FINAL_TYPE_WITH_CODE (EphySearchEngineManager, ephy_search_engine_manag
 enum {
   PROP_0,
   PROP_DEFAULT_ENGINE,
+  PROP_INCOGNITO_ENGINE,
   N_PROPS
 };
 
@@ -104,12 +106,17 @@ load_search_engines_from_settings (EphySearchEngineManager *manager)
   g_autoptr (GVariantIter) iter = NULL;
   GVariant *variant;
   g_autofree char *default_engine_name = g_settings_get_string (EPHY_SETTINGS_MAIN, EPHY_PREFS_DEFAULT_SEARCH_ENGINE);
+  g_autofree char *incognito_engine_name = g_settings_get_string (EPHY_SETTINGS_MAIN, EPHY_PREFS_INCOGNITO_SEARCH_ENGINE);
 
   g_settings_get (EPHY_SETTINGS_MAIN, EPHY_PREFS_SEARCH_ENGINES, "aa{sv}", &iter);
 
   while ((variant = g_variant_iter_next_value (iter))) {
-    GVariantDict dict;
-    const char *name, *url, *bang;
+    g_auto (GVariantDict) dict = {};
+    /* Make sure we keep our state clean and respect the non-NULL expectations. */
+    const char *name = "", *url = "", *bang = "";
+    /* These need to be NULL when there isn't a corresponding key in the dict. */
+    const char *suggestions_url = NULL;
+    const char *opensearch_url = NULL;
     g_autoptr (EphySearchEngine) search_engine = NULL;
 
     g_variant_dict_init (&dict, variant);
@@ -117,18 +124,18 @@ load_search_engines_from_settings (EphySearchEngineManager *manager)
     /* All of those checks are just to make sure we keep our state clean and
      * respect the non-NULL expectations.
      */
-    if (!g_variant_dict_lookup (&dict, "name", "&s", &name))
-      name = "";
-    if (!g_variant_dict_lookup (&dict, "url", "&s", &url))
-      url = "";
-    if (!g_variant_dict_lookup (&dict, "bang", "&s", &bang))
-      bang = "";
-    g_variant_dict_clear (&dict);
+    g_variant_dict_lookup (&dict, "name", "&s", &name);
+    g_variant_dict_lookup (&dict, "url", "&s", &url);
+    g_variant_dict_lookup (&dict, "bang", "&s", &bang);
+    g_variant_dict_lookup (&dict, "suggestions_url", "&s", &suggestions_url);
+    g_variant_dict_lookup (&dict, "opensearch_url", "&s", &opensearch_url);
 
     search_engine = g_object_new (EPHY_TYPE_SEARCH_ENGINE,
                                   "name", name,
                                   "url", url,
                                   "bang", bang,
+                                  "suggestions-url", suggestions_url,
+                                  "opensearch-url", opensearch_url,
                                   NULL);
     g_assert (EPHY_IS_SEARCH_ENGINE (search_engine));
 
@@ -145,6 +152,8 @@ load_search_engines_from_settings (EphySearchEngineManager *manager)
     ephy_search_engine_manager_add_engine (manager, search_engine);
     if (g_strcmp0 (ephy_search_engine_get_name (search_engine), default_engine_name) == 0)
       ephy_search_engine_manager_set_default_engine (manager, search_engine);
+    if (g_strcmp0 (ephy_search_engine_get_name (search_engine), incognito_engine_name) == 0)
+      ephy_search_engine_manager_set_incognito_engine (manager, search_engine);
 
     g_variant_unref (variant);
   }
@@ -156,6 +165,7 @@ load_search_engines_from_settings (EphySearchEngineManager *manager)
   if (G_UNLIKELY (manager->engines->len == 0)) {
     g_settings_reset (EPHY_SETTINGS_MAIN, EPHY_PREFS_SEARCH_ENGINES);
     g_settings_reset (EPHY_SETTINGS_MAIN, EPHY_PREFS_DEFAULT_SEARCH_ENGINE);
+    g_settings_reset (EPHY_SETTINGS_MAIN, EPHY_PREFS_INCOGNITO_SEARCH_ENGINE);
     load_search_engines_from_settings (manager);
 
     g_warning ("Having no search engine is forbidden. Resetting to default ones instead.");
@@ -165,6 +175,13 @@ load_search_engines_from_settings (EphySearchEngineManager *manager)
   if (G_UNLIKELY (!manager->default_engine)) {
     g_warning ("Could not find default search engine set in the gsettings within all available search engines! Setting the first one as fallback.");
     ephy_search_engine_manager_set_default_engine (manager, manager->engines->pdata[0]);
+    g_settings_set_string (EPHY_SETTINGS_MAIN, EPHY_PREFS_DEFAULT_SEARCH_ENGINE, ephy_search_engine_get_name (manager->engines->pdata[0]));
+  }
+
+  if (G_UNLIKELY (!manager->incognito_engine)) {
+    g_warning ("Could not find incognito search engine set in the gsettings within all available search engines! Setting the first one as fallback.");
+    ephy_search_engine_manager_set_incognito_engine (manager, manager->engines->pdata[0]);
+    g_settings_set_string (EPHY_SETTINGS_MAIN, EPHY_PREFS_INCOGNITO_SEARCH_ENGINE, ephy_search_engine_get_name (manager->engines->pdata[0]));
   }
 }
 
@@ -194,6 +211,9 @@ ephy_search_engine_manager_get_property (GObject    *object,
     case PROP_DEFAULT_ENGINE:
       g_value_take_object (value, ephy_search_engine_manager_get_default_engine (self));
       break;
+    case PROP_INCOGNITO_ENGINE:
+      g_value_take_object (value, ephy_search_engine_manager_get_incognito_engine (self));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -211,6 +231,9 @@ ephy_search_engine_manager_set_property (GObject      *object,
     case PROP_DEFAULT_ENGINE:
       ephy_search_engine_manager_set_default_engine (self, g_value_get_object (value));
       break;
+    case PROP_INCOGNITO_ENGINE:
+      ephy_search_engine_manager_set_incognito_engine (self, g_value_get_object (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -243,6 +266,13 @@ ephy_search_engine_manager_class_init (EphySearchEngineManagerClass *klass)
                          (G_PARAM_READWRITE |
                           G_PARAM_STATIC_STRINGS |
                           G_PARAM_EXPLICIT_NOTIFY));
+  properties [PROP_INCOGNITO_ENGINE] =
+    g_param_spec_object ("incognito-engine",
+                         NULL, NULL,
+                         EPHY_TYPE_SEARCH_ENGINE,
+                         (G_PARAM_READWRITE |
+                          G_PARAM_STATIC_STRINGS |
+                          G_PARAM_EXPLICIT_NOTIFY));
   g_object_class_install_properties (object_class, N_PROPS, properties);
 }
 
@@ -324,6 +354,43 @@ ephy_search_engine_manager_set_default_engine (EphySearchEngineManager *manager,
   g_object_notify_by_pspec (G_OBJECT (manager), properties[PROP_DEFAULT_ENGINE]);
 }
 
+/**
+ * ephy_search_engine_manager_get_incognito_engine:
+ *
+ * Returns: (transfer none): the incognito search engine for @manager.
+ */
+EphySearchEngine *
+ephy_search_engine_manager_get_incognito_engine (EphySearchEngineManager *manager)
+{
+  g_assert (EPHY_IS_SEARCH_ENGINE (manager->incognito_engine));
+
+  return manager->incognito_engine;
+}
+
+/**
+ * ephy_search_engine_manager_set_incognito_engine:
+ * @engine: (transfer none): the search engine to set as incognito engine for @manager.
+ *   This search engine must already be added to the search engine manager.
+ *
+ * Note that you must call ephy_search_engine_manager_save_to_settings() when
+ * appropriate to save it. It isn't done automatically because we don't save
+ * the search engines themselves on every change, as that would be pretty expensive
+ * when typing the information, so it's better if the incognito search engine and
+ * the search engines themselves are always kept in sync, in case there's an issue
+ * somewhere in the code where it doesn't save one part or another.
+ */
+void
+ephy_search_engine_manager_set_incognito_engine (EphySearchEngineManager *manager,
+                                                 EphySearchEngine        *engine)
+{
+  g_assert (EPHY_IS_SEARCH_ENGINE (engine));
+  /* Improper input validation if that happens in our code. */
+  g_assert (g_ptr_array_find (manager->engines, engine, NULL));
+
+  manager->incognito_engine = engine;
+  g_object_notify_by_pspec (G_OBJECT (manager), properties[PROP_INCOGNITO_ENGINE]);
+}
+
 /**
  * ephy_search_engine_manager_add_engine:
  * @engine: The search engine to add to @manager. @manager will take a reference
@@ -364,6 +431,12 @@ ephy_search_engine_manager_add_engine (EphySearchEngineManager *manager,
                               1);
 }
 
+/**
+ * ephy_search_engine_manager_delete_engine:
+ * @engine: The search engine to delete from @manager.
+ *
+ * Deletes search engine @engine from @manager.
+ */
 void
 ephy_search_engine_manager_delete_engine (EphySearchEngineManager *manager,
                                           EphySearchEngine        *engine)
@@ -402,6 +475,15 @@ ephy_search_engine_manager_delete_engine (EphySearchEngineManager *manager,
     ephy_search_engine_manager_set_default_engine (manager, manager->engines->pdata[0]);
   }
 
+  if (engine == manager->incognito_engine) {
+    g_assert (manager->engines->len != 0);
+
+    /* Just set the first search engine in the sorted array as new incognito search engine
+     * so we're sure we'll still have a valid incognito search engine at any time.
+     */
+    ephy_search_engine_manager_set_incognito_engine (manager, manager->engines->pdata[0]);
+  }
+
   /* Drop temporary ref. */
   g_object_unref (engine);
 
@@ -595,10 +677,34 @@ ephy_search_engine_manager_parse_bang_search (EphySearchEngineManager *manager,
   EphySearchEngine *engine = NULL;
   g_autofree char *no_bangs_query = parse_bang_query (manager, search, &engine);
 
-  if (no_bangs_query)
+  if (no_bangs_query) {
     return ephy_search_engine_build_search_address (engine, no_bangs_query);
-  else
+  }
+
+  return NULL;
+}
+
+/**
+ * ephy_search_engine_manager_parse_bang_suggestions:
+ *
+ * Same as ephy_search_engine_manager_parse_bang_search() but for the suggestions
+ * URL instead of the search URL.
+ */
+char *
+ephy_search_engine_manager_parse_bang_suggestions (EphySearchEngineManager  *manager,
+                                                   const char               *search,
+                                                   EphySearchEngine        **out_engine)
+{
+  EphySearchEngine *engine = NULL;
+  g_autofree char *no_bangs_query = parse_bang_query (manager, search, &engine);
+
+  if (no_bangs_query) {
+    if (out_engine)
+      *out_engine = engine;
+    return ephy_search_engine_build_suggestions_address (engine, no_bangs_query);
+  } else {
     return NULL;
+  }
 }
 
 /**
@@ -621,6 +727,7 @@ ephy_search_engine_manager_save_to_settings (EphySearchEngineManager *manager)
 
   while ((item = g_list_model_get_item (G_LIST_MODEL (manager), i++))) {
     g_autoptr (EphySearchEngine) engine = EPHY_SEARCH_ENGINE (item);
+    const char *suggestions_url, *opensearch_url;
     GVariantDict dict;
 
     g_assert (EPHY_IS_SEARCH_ENGINE (engine));
@@ -630,11 +737,16 @@ ephy_search_engine_manager_save_to_settings (EphySearchEngineManager *manager)
     g_variant_dict_insert (&dict, "url", "s", ephy_search_engine_get_url (engine));
     g_variant_dict_insert (&dict, "bang", "s", ephy_search_engine_get_bang (engine));
 
+    /* Let's not insert the OpenSearch-specific fields when they are not actually used. */
+    suggestions_url = ephy_search_engine_get_suggestions_url (engine);
+    opensearch_url = ephy_search_engine_get_opensearch_url (engine);
+    if (suggestions_url)
+      g_variant_dict_insert (&dict, "suggestions_url", "s", suggestions_url);
+    if (opensearch_url)
+      g_variant_dict_insert (&dict, "opensearch_url", "s", opensearch_url);
+
     g_variant_builder_add_value (&builder, g_variant_dict_end (&dict));
   }
   variant = g_variant_builder_end (&builder);
   g_settings_set_value (EPHY_SETTINGS_MAIN, EPHY_PREFS_SEARCH_ENGINES, variant);
-
-  g_settings_set_value (EPHY_SETTINGS_MAIN, EPHY_PREFS_DEFAULT_SEARCH_ENGINE,
-                        g_variant_new_string (ephy_search_engine_get_name (manager->default_engine)));
 }
