@@ -22,6 +22,7 @@
 #include "ephy-snapshot-service.h"
 
 #include "ephy-file-helpers.h"
+#include "ephy-pixbuf-utils.h"
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <stdio.h>
@@ -37,7 +38,7 @@ struct _EphySnapshotService {
   GHashTable *cache;
 };
 
-G_DEFINE_TYPE (EphySnapshotService, ephy_snapshot_service, G_TYPE_OBJECT)
+G_DEFINE_FINAL_TYPE (EphySnapshotService, ephy_snapshot_service, G_TYPE_OBJECT)
 
 typedef enum {
   SNAPSHOT_STALE,
@@ -83,41 +84,13 @@ thumbnail_filename (const char *uri)
   g_checksum_get_digest (checksum, digest, &digest_len);
   g_assert (digest_len == 16);
 
-  file = g_strconcat (g_checksum_get_string (checksum), ".png", NULL);
+  file = g_strconcat (g_checksum_get_string (checksum), ".jpg", NULL);
 
   g_checksum_free (checksum);
 
   return file;
 }
 
-static gboolean
-thumbnail_is_valid (GdkPixbuf  *pixbuf,
-                    const char *uri)
-{
-  const char *thumb_uri;
-
-  thumb_uri = gdk_pixbuf_get_option (pixbuf, "tEXt::Thumb::URI");
-  if (g_strcmp0 (uri, thumb_uri) != 0)
-    return FALSE;
-
-  return TRUE;
-}
-
-static gboolean
-validate_thumbnail_path (const char *path,
-                         const char *uri)
-{
-  GdkPixbuf *pixbuf;
-
-  pixbuf = gdk_pixbuf_new_from_file (path, NULL);
-  if (pixbuf == NULL || !thumbnail_is_valid (pixbuf, uri))
-    return FALSE;
-
-  g_object_unref (pixbuf);
-
-  return TRUE;
-}
-
 static char *
 thumbnail_directory (void)
 {
@@ -151,7 +124,6 @@ save_thumbnail (GdkPixbuf  *pixbuf,
   int tmp_fd;
   gboolean ret = FALSE;
   GError *error = NULL;
-  const char *width, *height;
 
   if (pixbuf == NULL)
     return FALSE;
@@ -169,26 +141,11 @@ save_thumbnail (GdkPixbuf  *pixbuf,
     goto out;
   close (tmp_fd);
 
-  width = gdk_pixbuf_get_option (pixbuf, "tEXt::Thumb::Image::Width");
-  height = gdk_pixbuf_get_option (pixbuf, "tEXt::Thumb::Image::Height");
-
   error = NULL;
-  if (width != NULL && height != NULL)
-    ret = gdk_pixbuf_save (pixbuf,
-                           tmp_path,
-                           "png", &error,
-                           "tEXt::Thumb::Image::Width", width,
-                           "tEXt::Thumb::Image::Height", height,
-                           "tEXt::Thumb::URI", uri,
-                           "tEXt::Software", "GNOME::Epiphany::ThumbnailFactory",
-                           NULL);
-  else
-    ret = gdk_pixbuf_save (pixbuf,
-                           tmp_path,
-                           "png", &error,
-                           "tEXt::Thumb::URI", uri,
-                           "tEXt::Software", "GNOME::Epiphany::ThumbnailFactory",
-                           NULL);
+  ret = gdk_pixbuf_save (pixbuf,
+                         tmp_path,
+                         "jpeg", &error,
+                         NULL);
 
   if (!ret)
     goto out;
@@ -223,7 +180,7 @@ ephy_snapshot_service_prepare_snapshot (GdkTexture *texture)
   if (!orig_width || !orig_height)
     return NULL;
 
-  snapshot = gdk_pixbuf_get_from_texture (texture);
+  snapshot = ephy_texture_to_pixbuf (texture, FALSE);
 
   if (orig_width < EPHY_THUMBNAIL_WIDTH ||
       orig_height < EPHY_THUMBNAIL_HEIGHT) {
@@ -239,6 +196,13 @@ ephy_snapshot_service_prepare_snapshot (GdkTexture *texture)
                                       EPHY_THUMBNAIL_WIDTH,
                                       new_height,
                                       GDK_INTERP_BILINEAR);
+
+    /* Crop image in case it exceeds our thumbnail height limit */
+    if (new_height > EPHY_THUMBNAIL_HEIGHT) {
+      GdkPixbuf *tmp = gdk_pixbuf_new_subpixbuf (scaled, 0, 0, EPHY_THUMBNAIL_WIDTH, EPHY_THUMBNAIL_HEIGHT);
+      g_clear_object (&scaled);
+      scaled = tmp;
+    }
   }
 
   g_object_unref (snapshot);
@@ -302,15 +266,13 @@ typedef struct {
   SnapshotPathCachedData *data;
 } CacheData;
 
-static gboolean
+static void
 idle_cache_snapshot_path (gpointer user_data)
 {
   CacheData *data = (CacheData *)user_data;
   g_hash_table_insert (data->cache, data->url, data->data);
   g_hash_table_unref (data->cache);
   g_free (data);
-
-  return G_SOURCE_REMOVE;
 }
 
 static void
@@ -326,7 +288,7 @@ cache_snapshot_data_in_idle (EphySnapshotService   *service,
   data->data = g_new (SnapshotPathCachedData, 1);
   data->data->path = g_strdup (path);
   data->data->freshness = freshness;
-  g_idle_add (idle_cache_snapshot_path, data);
+  g_idle_add_once (idle_cache_snapshot_path, data);
 }
 
 static void
@@ -591,14 +553,19 @@ get_snapshot_path_for_url_thread (GTask               *task,
                                   SnapshotAsyncData   *data,
                                   GCancellable        *cancellable)
 {
+  g_autoptr (GdkPixbuf) pixbuf = NULL;
+  g_autoptr (GError) error = NULL;
   char *path;
 
   path = thumbnail_path (data->url);
-  if (!validate_thumbnail_path (path, data->url)) {
+
+  pixbuf = gdk_pixbuf_new_from_file (path, &error);
+  if (pixbuf == NULL) {
     g_task_return_new_error (task,
                              EPHY_SNAPSHOT_SERVICE_ERROR,
                              EPHY_SNAPSHOT_SERVICE_ERROR_NOT_FOUND,
-                             "Snapshot for url \"%s\" not found in disk cache", data->url);
+                             "Snapshot for url \"%s\" not found in disk cache: %s",
+                             data->url, error->message);
     g_free (path);
     return;
   }
