@@ -39,7 +39,7 @@ ephy_history_service_initialize_hosts_table (EphyHistoryService *self)
                                   "url LONGVARCAR,"
                                   "title LONGVARCAR,"
                                   "visit_count INTEGER DEFAULT 0 NOT NULL,"
-                                  "zoom_level REAL DEFAULT 1.0)", &error);
+                                  "zoom_level REAL DEFAULT 0.0)", &error);
 
   if (error) {
     g_warning ("Could not create hosts table: %s", error->message);
@@ -50,7 +50,8 @@ ephy_history_service_initialize_hosts_table (EphyHistoryService *self)
 }
 
 void
-ephy_history_service_add_host_row (EphyHistoryService *self, EphyHistoryHost *host)
+ephy_history_service_add_host_row (EphyHistoryService *self,
+                                   EphyHistoryHost    *host)
 {
   EphySQLiteStatement *statement = NULL;
   GError *error = NULL;
@@ -90,10 +91,12 @@ ephy_history_service_add_host_row (EphyHistoryService *self, EphyHistoryHost *ho
 }
 
 void
-ephy_history_service_update_host_row (EphyHistoryService *self, EphyHistoryHost *host)
+ephy_history_service_update_host_row (EphyHistoryService *self,
+                                      EphyHistoryHost    *host)
 {
   EphySQLiteStatement *statement;
   GError *error = NULL;
+  gdouble zoom_level;
 
   g_assert (self->history_thread == g_thread_self ());
   g_assert (self->history_database != NULL);
@@ -107,10 +110,12 @@ ephy_history_service_update_host_row (EphyHistoryService *self, EphyHistoryHost
     return;
   }
 
+  zoom_level = host->zoom_level;
+
   if (ephy_sqlite_statement_bind_string (statement, 0, host->url, &error) == FALSE ||
       ephy_sqlite_statement_bind_string (statement, 1, host->title, &error) == FALSE ||
       ephy_sqlite_statement_bind_int (statement, 2, host->visit_count, &error) == FALSE ||
-      ephy_sqlite_statement_bind_double (statement, 3, host->zoom_level, &error) == FALSE ||
+      ephy_sqlite_statement_bind_double (statement, 3, zoom_level, &error) == FALSE ||
       ephy_sqlite_statement_bind_int (statement, 4, host->id, &error) == FALSE) {
     g_warning ("Could not modify host in hosts table: %s", error->message);
     g_error_free (error);
@@ -127,7 +132,9 @@ ephy_history_service_update_host_row (EphyHistoryService *self, EphyHistoryHost
 }
 
 EphyHistoryHost *
-ephy_history_service_get_host_row (EphyHistoryService *self, const gchar *host_string, EphyHistoryHost *host)
+ephy_history_service_get_host_row (EphyHistoryService *self,
+                                   const gchar        *host_string,
+                                   EphyHistoryHost    *host)
 {
   EphySQLiteStatement *statement = NULL;
   GError *error = NULL;
@@ -138,7 +145,7 @@ ephy_history_service_get_host_row (EphyHistoryService *self, const gchar *host_s
   if (host_string == NULL && host != NULL)
     host_string = host->url;
 
-  g_assert (host_string || host->id != -1);
+  g_assert (host_string || (host != NULL && host->id != -1));
 
   if (host != NULL && host->id != -1) {
     statement = ephy_sqlite_connection_create_statement (self->history_database,
@@ -176,7 +183,7 @@ ephy_history_service_get_host_row (EphyHistoryService *self, const gchar *host_s
   }
 
   if (host == NULL) {
-    host = ephy_history_host_new (NULL, NULL, 0, 1.0);
+    host = ephy_history_host_new (NULL, NULL, 0, 0.0);
   } else {
     if (host->url)
       g_free (host->url);
@@ -241,7 +248,8 @@ ephy_history_service_get_all_hosts (EphyHistoryService *self)
 }
 
 GList *
-ephy_history_service_find_host_rows (EphyHistoryService *self, EphyHistoryQuery *query)
+ephy_history_service_find_host_rows (EphyHistoryService *self,
+                                     EphyHistoryQuery   *query)
 {
   EphySQLiteStatement *statement = NULL;
   GList *substring;
@@ -342,7 +350,8 @@ ephy_history_service_find_host_rows (EphyHistoryService *self, EphyHistoryQuery
 
 /* Inspired from ephy-history.c */
 static GList *
-get_hostname_and_locations (const gchar *url, gchar **hostname)
+get_hostname_and_locations (const gchar  *url,
+                            gchar       **hostname)
 {
   GList *host_locations = NULL;
   char *scheme = NULL;
@@ -352,14 +361,14 @@ get_hostname_and_locations (const gchar *url, gchar **hostname)
     *hostname = ephy_string_get_host_name (url);
   }
   /* Build an host name */
-  if (scheme == NULL || *hostname == NULL) {
-    *hostname = g_strdup (_("Others"));
+  if (scheme != NULL && strcmp (scheme, "file") == 0) {
+    *hostname = g_strdup ("Local files");
+    host_locations = g_list_append (host_locations,
+                                    g_strdup (url));
+  } else if (scheme == NULL || *hostname == NULL) {
+    *hostname = g_strdup ("Others");
     host_locations = g_list_append (host_locations,
                                     g_strdup ("about:blank"));
-  }  else if (strcmp (scheme, "file") == 0) {
-    *hostname = g_strdup (_("Local files"));
-    host_locations = g_list_append (host_locations,
-                                    g_strdup ("file:///"));
   } else {
     char *location;
     char *tmp;
@@ -400,6 +409,7 @@ ephy_history_service_get_host_row_from_url (EphyHistoryService *self,
   EphyHistoryHost *host = NULL;
 
   host_locations = get_hostname_and_locations (url, &hostname);
+  g_assert (host_locations != NULL && hostname != NULL);
 
   for (l = host_locations; l != NULL; l = l->next) {
     host = ephy_history_service_get_host_row (self, l->data, NULL);
@@ -408,9 +418,8 @@ ephy_history_service_get_host_row_from_url (EphyHistoryService *self,
   }
 
   if (host == NULL) {
-    host = ephy_history_host_new (host_locations->data, hostname, 0, 1.0);
-    if (!self->read_only)
-      ephy_history_service_add_host_row (self, host);
+    host = ephy_history_host_new (host_locations->data, hostname, 0, 0.0);
+    ephy_history_service_add_host_row (self, host);
   }
 
   g_free (hostname);
@@ -475,10 +484,10 @@ ephy_history_service_delete_orphan_hosts (EphyHistoryService *self)
   g_assert (self->history_database != NULL);
 
   /* Where a JOIN would give us all hosts with urls associated, a LEFT
-     JOIN also gives us those hosts for which there are no urls.  By
-     means of urls.host == NULL we filter out anything else and
-     retrieve only the ids of the hosts without associated urls. Then,
-     we delete all these rows from the hosts table. */
+   *  JOIN also gives us those hosts for which there are no urls.  By
+   *  means of urls.host == NULL we filter out anything else and
+   *  retrieve only the ids of the hosts without associated urls. Then,
+   *  we delete all these rows from the hosts table. */
   ephy_sqlite_connection_execute (self->history_database,
                                   "DELETE FROM hosts WHERE hosts.id IN "
                                   "  (SELECT hosts.id FROM hosts LEFT JOIN urls "
