@@ -40,6 +40,8 @@ struct _EphyBookmarksManager {
 
   GSequence *bookmarks;
   GSequence *tags;
+  GSequence *bookmarks_order;
+  GSequence *tags_order;
 
   gchar *gvdb_filename;
 };
@@ -62,6 +64,9 @@ enum {
   BOOKMARK_TAG_REMOVED,
   TAG_CREATED,
   TAG_DELETED,
+  SORTED,
+  SYNCHRONIZABLE_DELETED,
+  SYNCHRONIZABLE_MODIFIED,
   LAST_SIGNAL
 };
 
@@ -117,7 +122,6 @@ ephy_bookmarks_manager_finalize (GObject *object)
 
   g_sequence_free (self->bookmarks);
   g_sequence_free (self->tags);
-
   g_free (self->gvdb_filename);
 
   G_OBJECT_CLASS (ephy_bookmarks_manager_parent_class)->finalize (object);
@@ -204,6 +208,21 @@ ephy_bookmarks_manager_class_init (EphyBookmarksManagerClass *klass)
                   NULL, NULL, NULL,
                   G_TYPE_NONE, 1,
                   G_TYPE_STRING);
+
+  signals[SORTED] =
+    g_signal_new ("sorted",
+                  EPHY_TYPE_BOOKMARKS_MANAGER,
+                  G_SIGNAL_RUN_LAST,
+                  0,
+                  NULL, NULL, NULL,
+                  G_TYPE_NONE, 1,
+                  G_TYPE_STRING);
+
+  signals[SYNCHRONIZABLE_DELETED] = g_signal_lookup ("synchronizable-deleted",
+                                                     EPHY_TYPE_SYNCHRONIZABLE_MANAGER);
+
+  signals[SYNCHRONIZABLE_MODIFIED] = g_signal_lookup ("synchronizable-modified",
+                                                      EPHY_TYPE_SYNCHRONIZABLE_MANAGER);
 }
 
 static void
@@ -219,6 +238,8 @@ ephy_bookmarks_manager_init (EphyBookmarksManager *self)
 
   self->bookmarks = g_sequence_new (g_object_unref);
   self->tags = g_sequence_new (g_free);
+  self->bookmarks_order = g_sequence_new (g_free);
+  self->tags_order = g_sequence_new (g_free);
 
   g_sequence_insert_sorted (self->tags,
                             g_strdup (EPHY_BOOKMARKS_FAVORITES_TAG),
@@ -234,26 +255,17 @@ ephy_bookmarks_manager_init (EphyBookmarksManager *self)
   }
 
   ephy_bookmarks_import (self, self->gvdb_filename, NULL);
+
+  ephy_bookmarks_manager_save (self, TRUE, TRUE, self->cancellable,
+                               (GAsyncReadyCallback)ephy_bookmarks_manager_save_warn_on_error_cb,
+                               NULL);
 }
 
-static void
-ephy_bookmark_manager_sort_list (EphyBookmark         *bookmark,
-                                 EphyBookmarksManager *self)
-{
-  guint n = g_list_model_get_n_items (G_LIST_MODEL (self));
-
-  g_sequence_sort (self->bookmarks, (GCompareDataFunc)ephy_bookmark_bookmarks_compare_func, bookmark);
-  g_list_model_items_changed (G_LIST_MODEL (self), 0, n, n);
-}
-
-
 static void
 bookmark_title_changed_cb (EphyBookmark         *bookmark,
                            GParamSpec           *pspec,
                            EphyBookmarksManager *self)
 {
-  ephy_bookmark_manager_sort_list (bookmark, self);
-
   g_signal_emit (self, signals[BOOKMARK_TITLE_CHANGED], 0, bookmark);
 }
 
@@ -262,8 +274,6 @@ bookmark_url_changed_cb (EphyBookmark         *bookmark,
                          GParamSpec           *pspec,
                          EphyBookmarksManager *self)
 {
-  ephy_bookmark_manager_sort_list (bookmark, self);
-
   g_signal_emit (self, signals[BOOKMARK_URL_CHANGED], 0, bookmark);
 }
 
@@ -272,9 +282,6 @@ bookmark_tag_added_cb (EphyBookmark         *bookmark,
                        const char           *tag,
                        EphyBookmarksManager *self)
 {
-  if (g_strcmp0 (tag, EPHY_BOOKMARKS_FAVORITES_TAG) == 0)
-    ephy_bookmark_manager_sort_list (bookmark, self);
-
   g_signal_emit (self, signals[BOOKMARK_TAG_ADDED], 0, bookmark, tag);
 }
 
@@ -283,9 +290,6 @@ bookmark_tag_removed_cb (EphyBookmark         *bookmark,
                          const char           *tag,
                          EphyBookmarksManager *self)
 {
-  if (g_strcmp0 (tag, EPHY_BOOKMARKS_FAVORITES_TAG) == 0)
-    ephy_bookmark_manager_sort_list (bookmark, self);
-
   g_signal_emit (self, signals[BOOKMARK_TAG_REMOVED], 0, bookmark, tag);
 }
 
@@ -343,7 +347,7 @@ ephy_bookmarks_manager_add_bookmark_internal (EphyBookmarksManager *self,
   }
 
   if (should_save)
-    ephy_bookmarks_manager_save (self, self->cancellable,
+    ephy_bookmarks_manager_save (self, FALSE, FALSE, self->cancellable,
                                  (GAsyncReadyCallback)ephy_bookmarks_manager_save_warn_on_error_cb,
                                  NULL);
 }
@@ -356,7 +360,7 @@ ephy_bookmarks_manager_add_bookmark (EphyBookmarksManager *self,
   g_assert (EPHY_IS_BOOKMARK (bookmark));
 
   ephy_bookmarks_manager_add_bookmark_internal (self, bookmark, TRUE);
-  g_signal_emit_by_name (self, "synchronizable-modified", bookmark, FALSE);
+  g_signal_emit (self, signals[SYNCHRONIZABLE_MODIFIED], 0, bookmark, FALSE);
 }
 
 void
@@ -373,12 +377,8 @@ ephy_bookmarks_manager_add_bookmarks (EphyBookmarksManager *self,
     EphyBookmark *bookmark = g_sequence_get (iter);
 
     ephy_bookmarks_manager_add_bookmark_internal (self, bookmark, FALSE);
-    g_signal_emit_by_name (self, "synchronizable-modified", bookmark, FALSE);
+    g_signal_emit (self, signals[SYNCHRONIZABLE_MODIFIED], 0, bookmark, FALSE);
   }
-
-  ephy_bookmarks_manager_save (self, self->cancellable,
-                               (GAsyncReadyCallback)ephy_bookmarks_manager_save_warn_on_error_cb,
-                               NULL);
 }
 
 static void
@@ -410,12 +410,11 @@ ephy_bookmarks_manager_remove_bookmark_internal (EphyBookmarksManager *self,
   g_list_model_items_changed (G_LIST_MODEL (self), position, 1, 0);
   g_signal_emit (self, signals[BOOKMARK_REMOVED], 0, bookmark);
 
-  ephy_bookmarks_manager_save (self, self->cancellable,
+  ephy_bookmarks_manager_save (self, FALSE, FALSE, self->cancellable,
                                (GAsyncReadyCallback)ephy_bookmarks_manager_save_warn_on_error_cb,
                                NULL);
 
   ephy_bookmarks_manager_unwatch_bookmark (self, bookmark);
-  g_object_unref (bookmark);
 }
 
 void
@@ -425,7 +424,7 @@ ephy_bookmarks_manager_remove_bookmark (EphyBookmarksManager *self,
   g_assert (EPHY_IS_BOOKMARKS_MANAGER (self));
   g_assert (EPHY_IS_BOOKMARK (bookmark));
 
-  g_signal_emit_by_name (self, "synchronizable-deleted", bookmark);
+  g_signal_emit (self, signals[SYNCHRONIZABLE_DELETED], 0, bookmark);
   ephy_bookmarks_manager_remove_bookmark_internal (self, bookmark);
 }
 
@@ -602,6 +601,160 @@ ephy_bookmarks_manager_get_tags (EphyBookmarksManager *self)
   return self->tags;
 }
 
+GSequence *
+ephy_bookmarks_manager_get_bookmarks_order (EphyBookmarksManager *self)
+{
+  g_assert (EPHY_IS_BOOKMARKS_MANAGER (self));
+
+  return self->bookmarks_order;
+}
+
+void
+ephy_bookmarks_manager_add_to_bookmarks_order (EphyBookmarksManager *self,
+                                               const char           *type,
+                                               const char           *item,
+                                               int                   index)
+{
+  GVariant *variant;
+
+  g_assert (EPHY_IS_BOOKMARKS_MANAGER (self));
+
+  variant = g_variant_new ("(ssi)", type, item, index);
+  g_sequence_append (self->bookmarks_order, variant);
+}
+
+void
+ephy_bookmarks_manager_clear_bookmarks_order (EphyBookmarksManager *self)
+{
+  g_assert (EPHY_IS_BOOKMARKS_MANAGER (self));
+
+  g_free (self->bookmarks_order);
+  self->bookmarks_order = g_sequence_new (g_free);
+}
+
+int
+sort_bookmarks_order (GVariant *variant_1,
+                      GVariant *variant_2)
+{
+  int index_1, index_2;
+
+  g_variant_get (variant_1, "(ssi)", NULL, NULL, &index_1);
+  g_variant_get (variant_2, "(ssi)", NULL, NULL, &index_2);
+
+  return index_1 - index_2;
+}
+
+void
+ephy_bookmarks_manager_sort_bookmarks_order (EphyBookmarksManager *self)
+{
+  g_sequence_sort (self->bookmarks_order, (GCompareDataFunc)sort_bookmarks_order, NULL);
+}
+
+GSequence *
+ephy_bookmarks_manager_get_tags_order (EphyBookmarksManager *self)
+{
+  g_assert (EPHY_IS_BOOKMARKS_MANAGER (self));
+
+  return self->tags_order;
+}
+
+GSequence *
+ephy_bookmarks_manager_tags_order_get_tag (EphyBookmarksManager *self,
+                                           const char           *tag)
+{
+  GSequence *urls = NULL;
+  GSequenceIter *iter;
+
+  g_assert (EPHY_IS_BOOKMARKS_MANAGER (self));
+
+  for (iter = g_sequence_get_begin_iter (self->tags_order);
+       !g_sequence_iter_is_end (iter);
+       iter = g_sequence_iter_next (iter)) {
+    GVariant *variant = g_sequence_get (iter);
+    const char *variant_tag;
+    GVariantIter *variant_iter;
+
+    g_variant_get (variant, "(sa(si))", &variant_tag, &variant_iter);
+
+    if (g_strcmp0 (variant_tag, tag) == 0) {
+      const char *url;
+
+      urls = g_sequence_new (g_free);
+      while (g_variant_iter_next (variant_iter, "(si)", &url, NULL))
+        g_sequence_append (urls, g_strdup (url));
+    }
+
+    g_variant_iter_free (variant_iter);
+    if (urls)
+      break;
+  }
+
+  return urls;
+}
+
+void
+ephy_bookmarks_manager_tags_order_clear_tag (EphyBookmarksManager *self,
+                                             const char           *tag)
+{
+  GSequenceIter *iter;
+
+  g_assert (EPHY_IS_BOOKMARKS_MANAGER (self));
+
+  for (iter = g_sequence_get_begin_iter (self->tags_order);
+       !g_sequence_iter_is_end (iter);
+       iter = g_sequence_iter_next (iter)) {
+    GVariant *variant = g_sequence_get (iter);
+    const char *variant_tag;
+
+    g_variant_get (variant, "(sa(si))", &variant_tag, NULL);
+
+    if (g_strcmp0 (variant_tag, tag) == 0) {
+      g_sequence_remove (iter);
+      return;
+    }
+  }
+}
+
+void
+ephy_bookmarks_manager_tags_order_add_tag (EphyBookmarksManager *self,
+                                           const char           *tag,
+                                           GSequence            *urls)
+{
+  GVariantBuilder builder;
+  GVariant *variant;
+  GSequenceIter *iter;
+
+  g_assert (EPHY_IS_BOOKMARKS_MANAGER (self));
+
+  g_variant_builder_init (&builder, G_VARIANT_TYPE ("(sa(si))"));
+  g_variant_builder_add (&builder, "s", tag);
+  g_variant_builder_open (&builder, G_VARIANT_TYPE ("a(si)"));
+
+  for (iter = g_sequence_get_begin_iter (urls);
+       !g_sequence_iter_is_end (iter);
+       iter = g_sequence_iter_next (iter)) {
+    const char *url = g_sequence_get (iter);
+
+    g_variant_builder_open (&builder, G_VARIANT_TYPE ("(si)"));
+    g_variant_builder_add (&builder, "s", url);
+    g_variant_builder_add (&builder, "i", g_sequence_iter_get_position (iter));
+    g_variant_builder_close (&builder);
+  }
+
+  g_variant_builder_close (&builder);
+  variant = g_variant_builder_end (&builder);
+  g_sequence_append (self->tags_order, variant);
+
+  g_sequence_free (urls);
+}
+
+void
+ephy_bookmarks_manager_tags_order_add_tag_variant (EphyBookmarksManager *self,
+                                                   GVariant             *variant)
+{
+  g_sequence_append (self->tags_order, variant);
+}
+
 void
 ephy_bookmarks_manager_save_warn_on_error_cb (GObject      *object,
                                               GAsyncResult *result,
@@ -643,6 +796,8 @@ bookmarks_export_cb (GObject      *source_object,
 
 void
 ephy_bookmarks_manager_save (EphyBookmarksManager *self,
+                             gboolean              with_bookmarks_order,
+                             gboolean              with_tags_order,
                              GCancellable         *cancellable,
                              GAsyncReadyCallback   callback,
                              gpointer              user_data)
@@ -651,8 +806,8 @@ ephy_bookmarks_manager_save (EphyBookmarksManager *self,
 
   task = g_task_new (self, cancellable, callback, user_data);
 
-  ephy_bookmarks_export (self, self->gvdb_filename,
-                         cancellable, bookmarks_export_cb, task);
+  ephy_bookmarks_export (self, self->gvdb_filename, with_bookmarks_order,
+                         with_tags_order, cancellable, bookmarks_export_cb, task);
 }
 
 gboolean
@@ -697,7 +852,7 @@ ephy_bookmarks_manager_save_sync (EphyBookmarksManager  *self,
   data->main_loop = g_main_loop_new (context, FALSE);
 
   g_main_context_push_thread_default (context);
-  ephy_bookmarks_manager_save (self, NULL, save_to_file_cb, data);
+  ephy_bookmarks_manager_save (self, FALSE, FALSE, NULL, save_to_file_cb, data);
   g_main_loop_run (data->main_loop);
   g_main_context_pop_thread_default (context);
 
@@ -810,7 +965,7 @@ synchronizable_manager_save (EphySynchronizableManager *manager,
 {
   EphyBookmarksManager *self = EPHY_BOOKMARKS_MANAGER (manager);
 
-  ephy_bookmarks_manager_save (self, self->cancellable,
+  ephy_bookmarks_manager_save (self, FALSE, FALSE, self->cancellable,
                                (GAsyncReadyCallback)ephy_bookmarks_manager_save_warn_on_error_cb,
                                NULL);
 }
@@ -900,7 +1055,7 @@ next:
   }
 
   /* Commit changes to file. */
-  ephy_bookmarks_manager_save (self, self->cancellable,
+  ephy_bookmarks_manager_save (self, FALSE, FALSE, self->cancellable,
                                (GAsyncReadyCallback)ephy_bookmarks_manager_save_warn_on_error_cb,
                                NULL);
   g_hash_table_unref (dont_upload);
@@ -978,7 +1133,7 @@ next:
   }
 
   /* Commit changes to file. */
-  ephy_bookmarks_manager_save (self, self->cancellable,
+  ephy_bookmarks_manager_save (self, FALSE, FALSE, self->cancellable,
                                (GAsyncReadyCallback)ephy_bookmarks_manager_save_warn_on_error_cb,
                                NULL);
 
