@@ -65,6 +65,8 @@ GSequence   *ephy_bookmarks_manager_get_tags                        (EphyBookmar
 gboolean     ephy_bookmarks_manager_save_sync                       (EphyBookmarksManager  *self,
                                                                      GError               **error);
 void         ephy_bookmarks_manager_save                            (EphyBookmarksManager  *self,
+                                                                     gboolean               with_bookmarks_order,
+                                                                     gboolean               with_tags_order,
                                                                      GCancellable          *cancellable,
                                                                      GAsyncReadyCallback    callback,
                                                                      gpointer               user_data);
@@ -72,6 +74,37 @@ gboolean     ephy_bookmarks_manager_save_finish                     (EphyBookmar
                                                                      GAsyncResult          *result,
                                                                      GError               **error);
 
+GSequence   *ephy_bookmarks_manager_get_bookmarks_order             (EphyBookmarksManager   *self);
+
+void         ephy_bookmarks_manager_sort_bookmarks_order            (EphyBookmarksManager   *self);
+
+void         ephy_bookmarks_manager_add_to_bookmarks_order          (EphyBookmarksManager   *self,
+                                                                     const char             *type,
+                                                                     const char             *item,
+                                                                     int                     index);
+
+void         ephy_bookmarks_manager_clear_bookmarks_order           (EphyBookmarksManager   *self);
+
+void         ephy_bookmarks_manager_add_to_bookmarks_order          (EphyBookmarksManager   *self,
+                                                                     const char             *tag,
+                                                                     const char             *bookmark_url,
+                                                                     int                     index);
+
+GSequence   *ephy_bookmarks_manager_get_tags_order                  (EphyBookmarksManager   *self);
+
+GSequence   *ephy_bookmarks_manager_tags_order_get_tag              (EphyBookmarksManager   *self,
+                                                                     const char             *tag);
+
+void         ephy_bookmarks_manager_tags_order_clear_tag            (EphyBookmarksManager   *self,
+                                                                     const char             *tag);
+
+void         ephy_bookmarks_manager_tags_order_add_tag               (EphyBookmarksManager   *self,
+                                                                      const char             *tag,
+                                                                      GSequence              *urls);
+
+void         ephy_bookmarks_manager_tags_order_add_tag_variant       (EphyBookmarksManager   *self,
+                                                                      GVariant               *variant);
+
 void          ephy_bookmarks_manager_save_warn_on_error_cb          (GObject               *object,
                                                                      GAsyncResult          *result,
                                                                      gpointer               user_data);
