@@ -288,65 +288,15 @@ update_suggestions_popover (EphyLocationEntry *self)
   }
 }
 
-static char *
-get_host_from_display_url (const char *decoded_uri)
-{
-  g_autofree char *copy = g_strdup (decoded_uri);
-  const char *protocol_end;
-  const char *authority_start;
-  const char *first_colon;
-  const char *first_slash;
-  const char *first_question_mark;
-  const char *first_octothorpe;
-  const char *first_hostname_terminator;
-  const char *last_at_sign;
-
-  /* Get host component without using GUri, such that the original URL is not
-   * modified in any way and the return value is guaranteed to be a substring of
-   * the input. (Notably, this avoids GUri performing punycode encoding.)
-   *
-   * The algorithm is based on WebKit's applyHostNameFunctionToURLString in
-   * URLHelpers.cpp (License: BSD-3-Clause).
-   */
-
-  protocol_end = strstr (copy, "://");
-  if (!protocol_end)
-    return NULL;
-
-  authority_start = protocol_end + strlen ("://");
-
-  first_colon = g_utf8_strchr (authority_start, -1, ':');
-  first_slash = g_utf8_strchr (authority_start, -1, '/');
-  first_question_mark = g_utf8_strchr (authority_start, -1, '?');
-  first_octothorpe = g_utf8_strchr (authority_start, -1, '#');
-
-  first_hostname_terminator = first_colon;
-  if (!first_hostname_terminator || (first_slash && first_slash < first_hostname_terminator))
-    first_hostname_terminator = first_slash;
-  if (!first_hostname_terminator || (first_question_mark && first_question_mark < first_hostname_terminator))
-    first_hostname_terminator = first_question_mark;
-  if (!first_hostname_terminator || (first_octothorpe && first_octothorpe < first_hostname_terminator))
-    first_hostname_terminator = first_octothorpe;
-
-  if (first_hostname_terminator)
-    ((char *)first_hostname_terminator)[0] = '\0';
-
-  last_at_sign = g_utf8_strrchr (authority_start, -1, '@');
-  if (last_at_sign)
-    return g_strdup (last_at_sign + 1);
-
-  return g_strdup (authority_start);
-}
-
 static guint
-get_port_from_display_url (const char *decoded_uri)
+get_port_from_url (const char *url)
 {
   g_autoptr (GUri) uri = NULL;
   g_autoptr (GError) error = NULL;
 
-  uri = g_uri_parse (decoded_uri, G_URI_FLAGS_PARSE_RELAXED, &error);
+  uri = g_uri_parse (url, G_URI_FLAGS_PARSE_RELAXED, &error);
   if (!uri)
-    g_warning ("Failed to parse URL %s: %s", decoded_uri, error->message);
+    g_warning ("Failed to parse URL %s: %s", url, error->message);
   return g_uri_get_port (uri);
 }
 
@@ -398,12 +348,12 @@ update_url_button_style (EphyLocationEntry *self)
     return;
   }
 
-  host = get_host_from_display_url (text);
-  if (!host) {
+  host = ephy_uri_get_decoded_host (text);
+  if (!host || !host[0]) {
     LOG ("Failed to get host component for URL %s", text);
     goto out;
   }
-  port = get_port_from_display_url (text);
+  port = get_port_from_url (text);
 
   base_domain = ephy_uri_get_base_domain (host);
   if (!base_domain) {
@@ -785,6 +735,9 @@ on_editable_changed (GtkEditable *editable,
     ephy_embed_set_typed_input (embed, text);
   }
 
+  if (g_strcmp0 (text, self->jump_tab) != 0)
+    g_clear_pointer (&self->jump_tab, g_free);
+
   if (self->insert_completion)
     self->idle_id = g_idle_add_full (G_PRIORITY_HIGH, calc_and_set_prefix, self, NULL);
 }
