@@ -26,16 +26,14 @@
 #include <gtk/gtk.h>
 
 #include "ephy-adaptive-mode.h"
-#include "ephy-bookmark-states.h"
-#include "ephy-password-popover.h"
-#include "ephy-permission-popover.h"
 #include "ephy-security-levels.h"
+#include "ephy-window.h"
 
 G_BEGIN_DECLS
 
 #define EPHY_TYPE_LOCATION_ENTRY (ephy_location_entry_get_type())
 
-G_DECLARE_FINAL_TYPE (EphyLocationEntry, ephy_location_entry, EPHY, LOCATION_ENTRY, GtkWidget)
+G_DECLARE_FINAL_TYPE (EphyLocationEntry, ephy_location_entry, EPHY, LOCATION_ENTRY, GtkBox)
 
 
 GtkWidget      *ephy_location_entry_new                        (void);
@@ -50,24 +48,6 @@ void            ephy_location_entry_undo_reset                 (EphyLocationEntr
 
 void            ephy_location_entry_grab_focus                 (EphyLocationEntry *entry);
 
-void            ephy_location_entry_set_bookmark_icon_state    (EphyLocationEntry     *entry,
-                                                                EphyBookmarkIconState  state);
-
-void            ephy_location_entry_set_lock_tooltip           (EphyLocationEntry *entry,
-                                                                const char        *tooltip);
-
-void            ephy_location_entry_set_password_popover       (EphyLocationEntry   *entry,
-                                                                EphyPasswordPopover *popover);
-
-void            ephy_location_entry_show_password_popover      (EphyLocationEntry *entry);
-
-void            ephy_location_entry_add_permission_popover     (EphyLocationEntry     *entry,
-                                                                EphyPermissionPopover *popover);
-
-void            ephy_location_entry_show_best_permission_popover (EphyLocationEntry *entry);
-
-void            ephy_location_entry_clear_permission_buttons   (EphyLocationEntry *entry);
-
 void            ephy_location_entry_set_reader_mode_visible    (EphyLocationEntry *entry,
                                                                 gboolean           visible);
 
@@ -95,4 +75,17 @@ GListModel     *ephy_location_entry_get_model                  (EphyLocationEntr
 void            ephy_location_entry_set_model                  (EphyLocationEntry *entry,
                                                                 GListModel        *model);
 
+void            ephy_location_entry_start_change_combined_stop_reload_state (EphyLocationEntry *self,
+                                                                             gboolean           loading);
+
+GtkWidget      *ephy_location_entry_get_opensearch_button      (EphyLocationEntry *entry);
+
+GtkWidget      *ephy_location_entry_get_site_menu_button      (EphyLocationEntry *self);
+
+gboolean        ephy_location_entry_has_focus                  (EphyLocationEntry *self);
+
+void            ephy_location_entry_set_position               (EphyLocationEntry *self,
+                                                                int                position);
+void            ephy_location_entry_set_zoom_level             (EphyLocationEntry *entry,
+                                                                char              *zoom_level);
 G_END_DECLS
