@@ -75,6 +75,8 @@ struct _EphyShell {
 
   gchar *open_notification_id;
   gboolean startup_finished;
+  gboolean checking_modified_forms;
+  int num_windows_with_modified_forms;
 
 #if USE_GRANITE
   GtkStyleProvider *style_provider;
@@ -172,6 +174,9 @@ ephy_shell_startup_continue (EphyShell               *shell,
     ephy_link_open (EPHY_LINK (window), NULL, NULL, EPHY_LINK_HOME_PAGE);
   }
 
+  if (ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (shell)) == EPHY_EMBED_SHELL_MODE_KIOSK)
+    gtk_window_fullscreen (GTK_WINDOW (active_window));
+
   shell->startup_finished = TRUE;
 }
 
@@ -295,18 +300,6 @@ show_preferences (GSimpleAction *action,
   window_cmd_show_preferences (NULL, NULL, EPHY_WINDOW (window));
 }
 
-static void
-show_shortcuts (GSimpleAction *action,
-                GVariant      *parameter,
-                gpointer       user_data)
-{
-  GtkWindow *window;
-
-  window = gtk_application_get_active_window (GTK_APPLICATION (ephy_shell));
-
-  window_cmd_show_shortcuts (NULL, NULL, EPHY_WINDOW (window));
-}
-
 static void
 show_help (GSimpleAction *action,
            GVariant      *parameter,
@@ -383,6 +376,53 @@ webextension_context_menu_action (GSimpleAction *action,
   ephy_web_extension_manager_handle_context_menu_action (manager, parameter);
 }
 
+static void
+close_all_tabs (GSimpleAction *action,
+                GVariant      *parameter,
+                gpointer       user_data)
+{
+  GtkWindow *window;
+
+  window = gtk_application_get_active_window (GTK_APPLICATION (ephy_shell));
+
+  window_cmd_close_all_tabs (NULL, NULL, window);
+}
+
+static void
+on_uninstall_web_web_app_response (GtkWidget *dialog,
+                                   char      *response,
+                                   gpointer   user_data)
+{
+  EphyShell *shell;
+  EphyWebApplication *web_app;
+
+  if (g_strcmp0 (response, "uninstall") != 0)
+    return;
+
+  shell = ephy_shell_get_default ();
+  web_app = ephy_shell_get_webapp (shell);
+
+  ephy_web_application_delete (web_app->id, NULL);
+  ephy_shell_try_quit (shell);
+}
+
+void
+uninstall_web_app (GSimpleAction *action,
+                   GVariant      *parameter,
+                   gpointer       user_data)
+{
+  AdwDialog *dialog = adw_alert_dialog_new (_("Uninstall Web App?"), _("This web appʼs data will be permanently deleted"));
+  GtkWidget *parent = GTK_WIDGET (gtk_application_get_active_window (GTK_APPLICATION (ephy_shell_get_default ())));
+
+  adw_alert_dialog_add_response (ADW_ALERT_DIALOG (dialog), "cancel", _("_Cancel"));
+  adw_alert_dialog_add_response (ADW_ALERT_DIALOG (dialog), "uninstall", _("_Uninstall"));
+  adw_alert_dialog_set_response_appearance (ADW_ALERT_DIALOG (dialog), "uninstall", ADW_RESPONSE_DESTRUCTIVE);
+  adw_alert_dialog_set_default_response (ADW_ALERT_DIALOG (dialog), "uninstall");
+
+  g_signal_connect (dialog, "response", G_CALLBACK (on_uninstall_web_web_app_response), NULL);
+  adw_dialog_present (dialog, parent);
+}
+
 static GActionEntry app_entries[] = {
   { "new-window", new_window, NULL, NULL, NULL },
   { "new-incognito", new_incognito_window, NULL, NULL, NULL },
@@ -394,7 +434,6 @@ static GActionEntry app_entries[] = {
   { "firefox-sync-dialog", show_firefox_sync, NULL, NULL, NULL },
   { "clear-data-view", show_clear_data_view, NULL, NULL, NULL},
   { "preferences", show_preferences, NULL, NULL, NULL },
-  { "shortcuts", show_shortcuts, NULL, NULL, NULL },
   { "help", show_help, NULL, NULL, NULL },
   { "about", show_about, NULL, NULL, NULL },
   { "quit", quit_application, NULL, NULL, NULL },
@@ -402,6 +441,7 @@ static GActionEntry app_entries[] = {
   { "launch-app", launch_app, "s", NULL, NULL },
   { "webextension-notification", webextension_action, "(ssi)", NULL, NULL },
   { "webextension-context-menu", webextension_context_menu_action, "(sss)", NULL, NULL },
+  { "close-all-tabs", close_all_tabs, NULL, NULL, NULL },
 };
 
 static GActionEntry non_incognito_extra_app_entries[] = {
@@ -416,6 +456,7 @@ static GActionEntry app_mode_app_entries[] = {
   { "about", show_about, NULL, NULL, NULL },
   { "quit", quit_application, NULL, NULL, NULL },
   { "run-in-background", NULL, NULL, "false", NULL},
+  { "uninstall-web-app", uninstall_web_app },
 };
 
 #if 0
@@ -498,6 +539,7 @@ set_accel_for_action (EphyShell   *shell,
   gtk_application_set_accels_for_action (GTK_APPLICATION (shell), detailed_action_name, accels);
 }
 
+
 static gboolean
 run_in_background_get_mapping (GValue   *value,
                                GVariant *variant,
@@ -595,7 +637,6 @@ ephy_shell_startup (GApplication *application)
     set_accel_for_action (shell, "app.reopen-closed-tab", "<Primary><Shift>t");
     set_accel_for_action (shell, "app.import-bookmarks", "<Primary><Shift>m");
     set_accel_for_action (shell, "app.export-bookmarks", "<Primary><Shift>x");
-    set_accel_for_action (shell, "app.shortcuts", "<Primary>question");
     set_accel_for_action (shell, "app.help", "F1");
   } else {
     shell->webapp = ephy_web_application_for_profile_directory (ephy_profile_dir (), EPHY_WEB_APP_NO_TMP_ICON);
@@ -604,6 +645,9 @@ ephy_shell_startup (GApplication *application)
                                      app_mode_app_entries, G_N_ELEMENTS (app_mode_app_entries),
                                      application);
 
+    action = g_action_map_lookup_action (G_ACTION_MAP (application), "shortcuts");
+    g_simple_action_set_enabled (G_SIMPLE_ACTION (action), FALSE);
+
     action = g_action_map_lookup_action (G_ACTION_MAP (application), "run-in-background");
     g_settings_bind_with_mapping (EPHY_SETTINGS_WEB_APP,
                                   EPHY_PREFS_WEB_APP_RUN_IN_BACKGROUND,
@@ -612,7 +656,7 @@ ephy_shell_startup (GApplication *application)
                                   G_SETTINGS_BIND_DEFAULT,
                                   run_in_background_get_mapping,
                                   run_in_background_set_mapping,
-                                  NULL,
+                                  shell,
                                   NULL);
   }
 
@@ -759,7 +803,7 @@ ephy_shell_add_platform_data (GApplication    *application,
 
   G_APPLICATION_CLASS (ephy_shell_parent_class)->add_platform_data (application, builder);
 
-  if (!app->local_startup_context)
+  if (!g_application_get_is_remote (application))
     return;
 
   /*
@@ -767,6 +811,7 @@ ephy_shell_add_platform_data (GApplication    *application,
    * ctx that are non-NULL.
    */
 
+  g_assert (app->local_startup_context);
   ctx = app->local_startup_context;
   if (ctx->startup_mode == 0 && !ctx->session_filename && !ctx->arguments)
     return;
@@ -834,6 +879,11 @@ ephy_shell_before_emit (GApplication *application,
             break;
         }
       }
+
+      /* Normally g_variant_iter_loop() frees this for us, but not if we break
+       * out of the loop and never call it again.
+       */
+      g_variant_unref (value);
       break;
     }
   }
@@ -1380,6 +1430,62 @@ ephy_shell_set_startup_context (EphyShell               *shell,
   shell->local_startup_context = ctx;
 }
 
+gboolean
+ephy_shell_get_checking_modified_forms (EphyShell *shell)
+{
+  g_assert (EPHY_IS_SHELL (shell));
+
+  return shell->checking_modified_forms;
+}
+
+void
+ephy_shell_set_checking_modified_forms (EphyShell *shell,
+                                        gboolean   is_checking)
+{
+  g_assert (EPHY_IS_SHELL (shell));
+
+  shell->checking_modified_forms = is_checking;
+}
+
+static void
+finish_closing_after_modified_forms_check (EphyShell *shell)
+{
+  GList *windows = gtk_application_get_windows (GTK_APPLICATION (shell));
+  EphySession *session = ephy_shell_get_session (shell);
+
+  if (session)
+    ephy_session_close (session);
+
+  shell->checking_modified_forms = FALSE;
+  while (windows) {
+    EphyWindow *window = EPHY_WINDOW (windows->data);
+
+    windows = windows->next;
+    ephy_window_handle_quit_with_modified_forms (window);
+  }
+  g_application_quit (G_APPLICATION (shell));
+}
+
+int
+ephy_shell_get_num_windows_with_modified_forms (EphyShell *shell)
+{
+  g_assert (EPHY_IS_SHELL (shell));
+
+  return shell->num_windows_with_modified_forms;
+}
+
+void
+ephy_shell_set_num_windows_with_modified_forms (EphyShell *shell,
+                                                int        windows)
+{
+  g_assert (EPHY_IS_SHELL (shell));
+
+  shell->num_windows_with_modified_forms = windows;
+
+  if (!shell->num_windows_with_modified_forms && shell->checking_modified_forms)
+    finish_closing_after_modified_forms_check (shell);
+}
+
 guint
 ephy_shell_get_n_windows (EphyShell *shell)
 {
@@ -1400,19 +1506,27 @@ ephy_shell_close_all_windows (EphyShell *shell)
 
   g_assert (EPHY_IS_SHELL (shell));
 
-  if (session)
-    ephy_session_close (session);
-
   windows = gtk_application_get_windows (GTK_APPLICATION (shell));
+  shell->checking_modified_forms = TRUE;
+  shell->num_windows_with_modified_forms = ephy_shell_get_n_windows (shell);
   while (windows) {
     EphyWindow *window = EPHY_WINDOW (windows->data);
 
     windows = windows->next;
 
-    if (ephy_window_close (window))
-      gtk_window_destroy (GTK_WINDOW (window));
-    else
+    /* Check if the window already has modified forms. This prevents
+     * showing the modified forms dialog multiple times, which could happen
+     * if the user attempts to quit multiple times.
+     */
+    if (ephy_window_get_has_modified_forms (window)) {
       retval = FALSE;
+      continue;
+    }
+
+    if (!ephy_window_can_close (window))
+      retval = FALSE;
+    else
+      shell->num_windows_with_modified_forms--;
   }
 
   if (shell->open_notification_id) {
@@ -1420,6 +1534,19 @@ ephy_shell_close_all_windows (EphyShell *shell)
     g_clear_pointer (&shell->open_notification_id, g_free);
   }
 
+  if (retval) {
+    if (session)
+      ephy_session_close (session);
+
+    windows = gtk_application_get_windows (GTK_APPLICATION (shell));
+    while (windows) {
+      EphyWindow *window = EPHY_WINDOW (windows->data);
+
+      windows = windows->next;
+      ephy_window_close (window);
+    }
+  }
+
   return retval;
 }
 
@@ -1456,8 +1583,8 @@ open_uris_data_new (EphyShell        *shell,
   data->session = session ? g_object_ref (session) : NULL;
   data->uris = g_strdupv ((char **)uris);
 
-  fullscreen_lockdown = g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
-                                                EPHY_PREFS_LOCKDOWN_FULLSCREEN);
+  fullscreen_lockdown = g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN, EPHY_PREFS_LOCKDOWN_FULLSCREEN) ||
+                        ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (shell)) == EPHY_EMBED_SHELL_MODE_KIOSK;
 
   if (startup_mode == EPHY_STARTUP_NEW_WINDOW && !fullscreen_lockdown) {
     data->window = ephy_window_new ();
@@ -1529,10 +1656,12 @@ ephy_shell_open_uris_idle (OpenURIsData *data)
 
     if (data->flags & EPHY_NEW_TAB_JUMP && mode != EPHY_EMBED_SHELL_MODE_TEST)
       gtk_window_present (GTK_WINDOW (data->window));
+
+    ephy_window_focus_location_entry (data->window);
   } else {
     ephy_web_view_load_new_tab_page (ephy_embed_get_web_view (embed));
     if (data->flags & EPHY_NEW_TAB_JUMP)
-      ephy_window_activate_location (data->window);
+      ephy_window_focus_location_entry (data->window);
   }
 
   if (!url_is_xpi) {
