@@ -20,7 +20,7 @@
 
 #pragma once
 
-#include "ephy-bookmark-states.h"
+#include "ephy-bookmarks-dialog.h"
 #include "ephy-bookmarks-manager.h"
 #include "ephy-embed.h"
 #include "ephy-location-controller.h"
@@ -60,14 +60,19 @@ void              ephy_window_load_url            (EphyWindow *window,
 void              ephy_window_set_zoom            (EphyWindow *window,
                                                    double zoom);
 
-void              ephy_window_activate_location   (EphyWindow *window);
+void              ephy_window_focus_location_entry (EphyWindow *window);
 void              ephy_window_location_search     (EphyWindow *window);
 const char       *ephy_window_get_location        (EphyWindow *window);
 
 GtkWidget        *ephy_window_get_header_bar      (EphyWindow *window);
 
+gboolean          ephy_window_can_close           (EphyWindow *window);
 gboolean          ephy_window_close               (EphyWindow *window);
 
+void              ephy_window_handle_quit_with_modified_forms (EphyWindow *window);
+
+guint             ephy_window_get_has_modified_forms (EphyWindow *window);
+
 EphyWindowChrome  ephy_window_get_chrome          (EphyWindow *window);
 
 EphyLocationController  *ephy_window_get_location_controller (EphyWindow *window);
@@ -94,9 +99,6 @@ gboolean          ephy_window_is_fullscreen              (EphyWindow *window);
 GActionGroup     *ephy_window_get_action_group           (EphyWindow  *window,
                                                           const char  *prefix);
 
-void              ephy_window_sync_bookmark_state        (EphyWindow            *window,
-                                                          EphyBookmarkIconState  state);
-
 guint64           ephy_window_get_uid                    (EphyWindow *window);
 
 gboolean          ephy_window_get_sidebar_shown          (EphyWindow *window);
@@ -106,6 +108,15 @@ void              ephy_window_switch_to_new_tab_toast    (EphyWindow *window,
 
 void              ephy_window_switch_to_new_tab          (EphyWindow *window);
 
+gboolean          ephy_window_get_show_sidebar           (EphyWindow *window);
+
+void              ephy_window_bookmark_removed_toast     (EphyWindow   *window,
+                                                          EphyBookmark *bookmark,
+                                                          AdwToast     *toast);
+
 void              ephy_window_toggle_bookmarks           (EphyWindow *window);
 
+void              ephy_window_show_toast                 (EphyWindow *window,
+                                                          const char *text);
+
 G_END_DECLS
