@@ -25,6 +25,7 @@
 #include <ctype.h>
 
 typedef struct {
+  GtkWidget *toast_overlay;
   GtkWidget *box;
   GtkWidget *child;
   GtkWidget *clear_button;
@@ -34,6 +35,7 @@ typedef struct {
   GtkWidget *stack;
   GtkWidget *empty_page;
   GtkWidget *spinner;
+  GtkWidget *options_menu_button;
 
   gboolean is_loading : 1;
   gboolean has_data : 1;
@@ -339,13 +341,6 @@ ephy_data_view_class_init (EphyDataViewClass *klass)
 
   g_object_class_install_properties (object_class, LAST_PROP, obj_properties);
 
-  /**
-   * EphyLocationEntry::user-changed:
-   * @entry: the object on which the signal is emitted
-   *
-   * Emitted when the user changes the contents of the internal #GtkEntry
-   *
-   */
   signals[CLEAR_BUTTON_CLICKED] =
     g_signal_new ("clear-button-clicked",
                   G_OBJECT_CLASS_TYPE (klass),
@@ -358,6 +353,7 @@ ephy_data_view_class_init (EphyDataViewClass *klass)
   gtk_widget_class_set_template_from_resource (widget_class,
                                                "/org/gnome/epiphany/gtk/data-view.ui");
 
+  gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, toast_overlay);
   gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, box);
   gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, clear_button);
   gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, empty_page);
@@ -366,6 +362,7 @@ ephy_data_view_class_init (EphyDataViewClass *klass)
   gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, search_entry);
   gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, spinner);
   gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, stack);
+  gtk_widget_class_bind_template_child_private (widget_class, EphyDataView, options_menu_button);
 
   gtk_widget_class_bind_template_callback (widget_class, on_clear_button_clicked);
   gtk_widget_class_bind_template_callback (widget_class, on_search_entry_changed);
@@ -421,6 +418,18 @@ ephy_data_view_buildable_init (GtkBuildableIface *iface)
   iface->add_child = ephy_data_view_add_child;
 }
 
+AdwToastOverlay *
+ephy_data_view_get_toast_overlay (EphyDataView *self)
+{
+  EphyDataViewPrivate *priv;
+
+  g_assert (EPHY_IS_DATA_VIEW (self));
+
+  priv = ephy_data_view_get_instance_private (self);
+
+  return ADW_TOAST_OVERLAY (priv->toast_overlay);
+}
+
 const gchar *
 ephy_data_view_get_clear_button_label (EphyDataView *self)
 {
@@ -624,3 +633,13 @@ ephy_data_view_get_search_text (EphyDataView *self)
 
   return priv->search_text;
 }
+
+void
+ephy_data_view_set_options_menu (EphyDataView *self,
+                                 GMenuModel   *model)
+{
+  EphyDataViewPrivate *priv = ephy_data_view_get_instance_private (self);
+
+  gtk_widget_set_visible (priv->options_menu_button, model != NULL);
+  gtk_menu_button_set_menu_model (GTK_MENU_BUTTON (priv->options_menu_button), model);
+}
