@@ -33,15 +33,22 @@
 #include "ephy-settings.h"
 #include "ephy-web-extension.h"
 #include "extension-view.h"
-#include "passwords-view.h"
+#include "prefs-features-page.h"
 #include "prefs-general-page.h"
 #include "prefs-extensions-page.h"
+#include "webapp-additional-urls-dialog.h"
 
 struct _EphyPrefsDialog {
   AdwPreferencesDialog parent_instance;
 
   PrefsGeneralPage *general_page;
   GtkWidget *extensions_page;
+
+  EphyWindow *parent_window;
+
+#if !TECH_PREVIEW
+  GtkWidget *features_page;
+#endif
 };
 
 G_DEFINE_FINAL_TYPE (EphyPrefsDialog, ephy_prefs_dialog, ADW_TYPE_PREFERENCES_DIALOG)
@@ -50,15 +57,22 @@ static gboolean
 on_closed (EphyPrefsDialog *prefs_dialog)
 {
   prefs_general_page_on_pd_close_request (prefs_dialog->general_page);
-
-  /* To avoid any unnecessary IO when typing changes in the search engine
-   * list row's entries, only save when closing the prefs dialog.
-   */
-  ephy_search_engine_manager_save_to_settings (ephy_embed_shell_get_search_engine_manager (ephy_embed_shell_get_default ()));
-
   return GDK_EVENT_PROPAGATE;
 }
 
+EphyWindow *
+ephy_prefs_dialog_get_parent_window (EphyPrefsDialog *prefs_dialog)
+{
+  return prefs_dialog->parent_window;
+}
+
+void
+ephy_prefs_dialog_set_parent_window (EphyPrefsDialog *prefs_dialog,
+                                     EphyWindow      *window)
+{
+  prefs_dialog->parent_window = window;
+}
+
 void
 ephy_prefs_dialog_show_clear_data_view (EphyPrefsDialog *prefs_dialog)
 {
@@ -72,19 +86,19 @@ ephy_prefs_dialog_show_clear_data_view (EphyPrefsDialog *prefs_dialog)
 }
 
 static void
-on_passwords_row_activated (GtkWidget       *privacy_page,
-                            EphyPrefsDialog *prefs_dialog)
+on_autofill_row_activated (GtkWidget       *privacy_page,
+                           EphyPrefsDialog *prefs_dialog)
 {
-  AdwNavigationPage *page = g_object_new (EPHY_TYPE_PASSWORDS_VIEW, NULL);
+  AdwNavigationPage *page = g_object_new (EPHY_TYPE_AUTOFILL_VIEW, NULL);
 
   adw_preferences_dialog_push_subpage (ADW_PREFERENCES_DIALOG (prefs_dialog), page);
 }
 
 static void
-on_autofill_row_activated (GtkWidget       *privacy_page,
-                           EphyPrefsDialog *prefs_dialog)
+on_manage_webapp_additional_urls_row_activated (GtkWidget       *privacy_page,
+                                                EphyPrefsDialog *prefs_dialog)
 {
-  AdwNavigationPage *page = g_object_new (EPHY_TYPE_AUTOFILL_VIEW, NULL);
+  AdwNavigationPage *page = ADW_NAVIGATION_PAGE (ephy_webapp_additional_urls_dialog_new ());
 
   adw_preferences_dialog_push_subpage (ADW_PREFERENCES_DIALOG (prefs_dialog), page);
 }
@@ -125,6 +139,24 @@ sync_extensions (EphyPrefsDialog *self)
   }
 }
 
+#if !TECH_PREVIEW
+static void
+sync_features (EphyPrefsDialog *self)
+{
+  gboolean enable_features = g_settings_get_boolean (EPHY_SETTINGS_UI, EPHY_PREFS_UI_WEBKIT_FEATURES_PANEL);
+
+  if (enable_features && !self->features_page) {
+    self->features_page = g_object_new (EPHY_TYPE_PREFS_FEATURES_PAGE, "dialog", self, NULL);
+    adw_preferences_dialog_add (ADW_PREFERENCES_DIALOG (self),
+                                ADW_PREFERENCES_PAGE (self->features_page));
+  } else if (self->features_page) {
+    adw_preferences_dialog_remove (ADW_PREFERENCES_DIALOG (self),
+                                   ADW_PREFERENCES_PAGE (self->features_page));
+    self->features_page = NULL;
+  }
+}
+#endif
+
 static void
 ephy_prefs_dialog_class_init (EphyPrefsDialogClass *klass)
 {
@@ -138,8 +170,8 @@ ephy_prefs_dialog_class_init (EphyPrefsDialogClass *klass)
   /* Template file callbacks */
   gtk_widget_class_bind_template_callback (widget_class, on_closed);
   gtk_widget_class_bind_template_callback (widget_class, on_autofill_row_activated);
-  gtk_widget_class_bind_template_callback (widget_class, on_passwords_row_activated);
   gtk_widget_class_bind_template_callback (widget_class, on_clear_data_row_activated);
+  gtk_widget_class_bind_template_callback (widget_class, on_manage_webapp_additional_urls_row_activated);
 }
 
 static void
@@ -147,10 +179,23 @@ ephy_prefs_dialog_init (EphyPrefsDialog *dialog)
 {
   gtk_widget_init_template (GTK_WIDGET (dialog));
 
+  gtk_widget_set_size_request (GTK_WIDGET (dialog), 360, 200);
   sync_extensions (dialog);
   g_signal_connect_object (EPHY_SETTINGS_WEB,
                            "changed::" EPHY_PREFS_WEB_ENABLE_WEBEXTENSIONS,
                            G_CALLBACK (sync_extensions),
                            dialog,
                            G_CONNECT_SWAPPED);
+
+#if TECH_PREVIEW
+  adw_preferences_dialog_add (ADW_PREFERENCES_DIALOG (dialog),
+                              g_object_new (EPHY_TYPE_PREFS_FEATURES_PAGE, "dialog", dialog, NULL));
+#else
+  sync_features (dialog);
+  g_signal_connect_object (EPHY_SETTINGS_UI,
+                           "changed::" EPHY_PREFS_UI_WEBKIT_FEATURES_PANEL,
+                           G_CALLBACK (sync_features),
+                           dialog,
+                           G_CONNECT_SWAPPED);
+#endif
 }
