@@ -27,6 +27,7 @@
 #include "ephy-lib-type-builtins.h"
 #include "ephy-settings.h"
 #include "ephy-shell.h"
+#include "ephy-zoom.h"
 
 #include <math.h>
 
@@ -50,6 +51,7 @@ struct _PrefsAppearancePage {
   GtkWidget *js_row;
   GtkWidget *js_edit_button;
   GtkWidget *default_zoom_row;
+  GtkFontDialog *font_dialog;
 
   GCancellable *cancellable;
 };
@@ -144,6 +146,27 @@ reader_color_scheme_set_mapping (const GValue       *value,
   }
 }
 
+static gboolean
+font_desc_get_mapping (GValue   *value,
+                       GVariant *variant,
+                       gpointer  user_data)
+{
+  const char *font_string = g_variant_get_string (variant, NULL);
+  g_autoptr (PangoFontDescription) desc = pango_font_description_from_string (font_string);
+  g_value_take_boxed (value, g_steal_pointer (&desc));
+  return TRUE;
+}
+
+static GVariant *
+font_desc_set_mapping (const GValue       *value,
+                       const GVariantType *expected_type,
+                       gpointer            user_data)
+{
+  PangoFontDescription *desc = g_value_get_boxed (value);
+  g_autofree char *font_string = pango_font_description_to_string (desc);
+  return g_variant_new_string (font_string);
+}
+
 static void
 css_file_created_cb (GObject      *source,
                      GAsyncResult *result,
@@ -164,7 +187,7 @@ css_file_created_cb (GObject      *source,
     }
   }
 
-  ephy_file_launch_uri_handler (file, "text/css", gtk_widget_get_display (GTK_WIDGET (page)), EPHY_FILE_LAUNCH_URI_HANDLER_FILE, EPHY_FILE_LAUNCH_URI_HANDLER_FLAGS_NONE);
+  ephy_file_launch_uri_handler (file, "text/css", gtk_widget_get_display (GTK_WIDGET (page)), EPHY_FILE_LAUNCH_URI_HANDLER_FILE);
 }
 
 static void
@@ -200,7 +223,7 @@ js_file_created_cb (GObject      *source,
     }
   }
 
-  ephy_file_launch_uri_handler (file, "text/javascript", gtk_widget_get_display (GTK_WIDGET (page)), EPHY_FILE_LAUNCH_URI_HANDLER_FILE, EPHY_FILE_LAUNCH_URI_HANDLER_FLAGS_NONE);
+  ephy_file_launch_uri_handler (file, "text/javascript", gtk_widget_get_display (GTK_WIDGET (page)), EPHY_FILE_LAUNCH_URI_HANDLER_FILE);
 }
 
 static void
@@ -216,29 +239,14 @@ js_edit_button_clicked_cb (GtkWidget           *button,
   g_file_create_async (g_steal_pointer (&js_file), G_FILE_CREATE_NONE, G_PRIORITY_DEFAULT, appearance_page->cancellable, js_file_created_cb, appearance_page);
 }
 
-static gboolean
-on_default_zoom_row_output (AdwSpinRow *spin,
-                            gpointer    user_data)
-{
-  g_autofree gchar *text = NULL;
-  gdouble value;
-
-  value = (int)adw_spin_row_get_value (spin);
-  text = g_strdup_printf ("%.f%%", value);
-  gtk_editable_set_text (GTK_EDITABLE (spin), text);
-
-  return TRUE;
-}
-
 static void
-on_default_zoom_row_changed (AdwSpinRow *spin,
-                             gpointer    user_data)
+on_zoom_selected (GtkWidget  *row,
+                  GParamSpec *pspec,
+                  gpointer    user_data)
 {
-  gdouble value;
+  int selected = adw_combo_row_get_selected (ADW_COMBO_ROW (row));
 
-  value = adw_spin_row_get_value (spin);
-  value = round (value) / 100;
-  g_settings_set_double (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_DEFAULT_ZOOM_LEVEL, value);
+  g_settings_set_double (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_DEFAULT_ZOOM_LEVEL, ephy_zoom_get_value (selected));
 }
 
 static void
@@ -256,23 +264,37 @@ setup_appearance_page (PrefsAppearancePage *appearance_page)
                    "enable-expansion",
                    G_SETTINGS_BIND_INVERT_BOOLEAN);
 
-  g_settings_bind (web_settings,
-                   EPHY_PREFS_WEB_SANS_SERIF_FONT,
-                   appearance_page->sans_fontbutton,
-                   "font",
-                   G_SETTINGS_BIND_DEFAULT);
+  g_settings_bind_with_mapping (web_settings,
+                                EPHY_PREFS_WEB_SANS_SERIF_FONT,
+                                appearance_page->sans_fontbutton,
+                                "font-desc",
+                                G_SETTINGS_BIND_DEFAULT,
+                                font_desc_get_mapping,
+                                font_desc_set_mapping,
+                                NULL, NULL);
 
-  g_settings_bind (web_settings,
-                   EPHY_PREFS_WEB_SERIF_FONT,
-                   appearance_page->serif_fontbutton,
-                   "font",
-                   G_SETTINGS_BIND_DEFAULT);
+  g_settings_bind_with_mapping (web_settings,
+                                EPHY_PREFS_WEB_SERIF_FONT,
+                                appearance_page->serif_fontbutton,
+                                "font-desc",
+                                G_SETTINGS_BIND_DEFAULT,
+                                font_desc_get_mapping,
+                                font_desc_set_mapping,
+                                NULL, NULL);
 
-  g_settings_bind (web_settings,
-                   EPHY_PREFS_WEB_MONOSPACE_FONT,
-                   appearance_page->mono_fontbutton,
-                   "font",
-                   G_SETTINGS_BIND_DEFAULT);
+  g_settings_bind_with_mapping (web_settings,
+                                EPHY_PREFS_WEB_MONOSPACE_FONT,
+                                appearance_page->mono_fontbutton,
+                                "font-desc",
+                                G_SETTINGS_BIND_DEFAULT,
+                                font_desc_get_mapping,
+                                font_desc_set_mapping,
+                                NULL, NULL);
+
+  appearance_page->font_dialog = gtk_font_dialog_new ();
+  gtk_font_dialog_button_set_dialog (GTK_FONT_DIALOG_BUTTON (appearance_page->sans_fontbutton), appearance_page->font_dialog);
+  gtk_font_dialog_button_set_dialog (GTK_FONT_DIALOG_BUTTON (appearance_page->serif_fontbutton), appearance_page->font_dialog);
+  gtk_font_dialog_button_set_dialog (GTK_FONT_DIALOG_BUTTON (appearance_page->mono_fontbutton), appearance_page->font_dialog);
 
   /* ======================================================================== */
   /* ========================== Reader Mode ================================= */
@@ -336,8 +358,10 @@ setup_appearance_page (PrefsAppearancePage *appearance_page)
                     G_CALLBACK (js_edit_button_clicked_cb),
                     appearance_page);
 
-  adw_spin_row_set_value (ADW_SPIN_ROW (appearance_page->default_zoom_row),
-                          g_settings_get_double (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_DEFAULT_ZOOM_LEVEL) * 100);
+  adw_combo_row_set_selected (ADW_COMBO_ROW (appearance_page->default_zoom_row),
+                              ephy_zoom_get_index (g_settings_get_double (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_DEFAULT_ZOOM_LEVEL)));
+
+  g_signal_connect (appearance_page->default_zoom_row, "notify::selected", G_CALLBACK (on_zoom_selected), appearance_page);
 }
 
 static void
@@ -350,6 +374,8 @@ prefs_appearance_page_dispose (GObject *object)
     g_clear_object (&page->cancellable);
   }
 
+  g_clear_object (&page->font_dialog);
+
   G_OBJECT_CLASS (prefs_appearance_page_parent_class)->dispose (object);
 }
 
@@ -385,8 +411,6 @@ prefs_appearance_page_class_init (PrefsAppearancePageClass *klass)
   /* Signals */
   gtk_widget_class_bind_template_callback (widget_class, reader_font_style_get_name);
   gtk_widget_class_bind_template_callback (widget_class, reader_color_scheme_get_name);
-  gtk_widget_class_bind_template_callback (widget_class, on_default_zoom_row_output);
-  gtk_widget_class_bind_template_callback (widget_class, on_default_zoom_row_changed);
 }
 
 static void
