@@ -21,25 +21,12 @@
 
 G_BEGIN_DECLS
 
-gboolean adw_widget_focus_child (GtkWidget        *widget,
-                                 GtkDirectionType  direction);
-
-gboolean adw_widget_grab_focus_self  (GtkWidget *widget);
-gboolean adw_widget_grab_focus_child (GtkWidget *widget);
-
 void adw_widget_compute_expand (GtkWidget *widget,
                                 gboolean  *hexpand_p,
                                 gboolean  *vexpand_p);
 
-void adw_widget_compute_expand_horizontal_only (GtkWidget *widget,
-                                                gboolean  *hexpand_p,
-                                                gboolean  *vexpand_p);
-
 GtkSizeRequestMode adw_widget_get_request_mode (GtkWidget *widget);
 
-gboolean adw_widget_lookup_color (GtkWidget  *widget,
-                                  const char *name,
-                                  GdkRGBA    *rgba);
 
 G_END_DECLS
 
