@@ -28,9 +28,8 @@
 static WebKitCookieManager *
 get_cookie_manager (void)
 {
-  WebKitWebContext *web_context = ephy_embed_shell_get_web_context (ephy_embed_shell_get_default ());
-  WebKitWebsiteDataManager *data_manager = webkit_web_context_get_website_data_manager (web_context);
-  return webkit_website_data_manager_get_cookie_manager (data_manager);
+  WebKitNetworkSession *network_session = ephy_embed_shell_get_network_session (ephy_embed_shell_get_default ());
+  return webkit_network_session_get_cookie_manager (network_session);
 }
 
 static const char *
@@ -63,11 +62,7 @@ static void
 add_cookie_to_json (JsonBuilder *builder,
                     SoupCookie  *cookie)
 {
-#if SOUP_CHECK_VERSION (2, 99, 4)
   GDateTime *expires = soup_cookie_get_expires (cookie);
-#else
-  SoupDate *expires = soup_cookie_get_expires (cookie);
-#endif
 
   json_builder_begin_object (builder);
   json_builder_set_member_name (builder, "name");
@@ -86,11 +81,7 @@ add_cookie_to_json (JsonBuilder *builder,
   json_builder_add_string_value (builder, samesite_to_string (soup_cookie_get_same_site_policy (cookie)));
   if (expires) {
     json_builder_set_member_name (builder, "expirationDate");
-#if SOUP_CHECK_VERSION (2, 99, 4)
     json_builder_add_int_value (builder, g_date_time_to_unix (expires));
-#else
-    json_builder_add_int_value (builder, soup_date_to_time_t (expires));
-#endif
   }
   json_builder_end_object (builder);
 }
@@ -312,7 +303,7 @@ cookies_handler_set (EphyWebExtensionSender *sender,
     return;
   }
 
-  parsed_uri = g_uri_parse (url, G_URI_FLAGS_ENCODED_PATH | G_URI_FLAGS_ENCODED_QUERY | G_URI_FLAGS_SCHEME_NORMALIZE, &error);
+  parsed_uri = g_uri_parse (url, G_URI_FLAGS_PARSE_RELAXED | G_URI_FLAGS_ENCODED_PATH | G_URI_FLAGS_ENCODED_QUERY | G_URI_FLAGS_SCHEME_NORMALIZE, &error);
   if (error) {
     g_task_return_new_error (task, WEB_EXTENSION_ERROR, WEB_EXTENSION_ERROR_INVALID_ARGUMENT, "cookies.set(): Failed to parse URI '%s': %s", url, error->message);
     return;
