@@ -245,7 +245,16 @@ void window_cmd_switch_new_tab                  (GSimpleAction *action,
 void window_cmd_privacy_report                  (GSimpleAction *action,
                                                  GVariant      *parameter,
                                                  gpointer       user_data);
+void window_cmd_passwords                       (GSimpleAction *action,
+                                                 GVariant      *parameter,
+                                                 gpointer       user_data);
 void window_cmd_export_passwords                (GSimpleAction *action,
                                                  GVariant      *parameter,
                                                  gpointer       user_data);
+void window_cmd_close_all_tabs                  (GSimpleAction *action,
+                                                 GVariant      *parameter,
+                                                 gpointer       user_data);
+void window_cmd_security_and_permissions        (GSimpleAction *action,
+                                                 GVariant      *parameter,
+                                                 gpointer       user_data);
 G_END_DECLS
